@@ -1,7 +1,7 @@
 /* daemon.c -- Server daemon
- * Created: Fri Feb 28 18:17:56 1997 by faith@cs.unc.edu
- * Revised: Sun Mar  1 07:01:58 1998 by faith@acm.org
- * Copyright 1997, 1998 Rickard E. Faith (faith@acm.org)
+ * Created: Fri Feb 28 18:17:56 1997 by faith@dict.org
+ * Revised: Fri Dec 22 06:04:36 2000 by faith@dict.org
+ * Copyright 1997, 1998, 1999, 2000 Rickard E. Faith (faith@dict.org)
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -17,7 +17,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 675 Mass Ave, Cambridge, MA 02139, USA.
  * 
- * $Id: daemon.c,v 1.28 1998/03/01 12:05:28 faith Exp $
+ * $Id: daemon.c,v 1.32 2000/12/22 14:15:25 faith Exp $
  * 
  */
 
@@ -524,12 +524,15 @@ static int daemon_count_matches( lst_List list )
 {
    lst_Position p;
    dictWord     *dw;
-   const char   *previous = NULL;
+   const char   *prevword = NULL;
+   dictDatabase *prevdb   = NULL;
    int          count     = 0;
    
    LST_ITERATE(list,p,dw) {
-      if (previous && !strcmp(previous,dw->word)) continue;
-      previous = dw->word;
+      if (prevdb == dw->database && prevword && !strcmp(prevword,dw->word))
+	  continue;
+      prevword = dw->word;
+      prevdb   = dw->database;
       ++count;
    }
    return count;
@@ -539,12 +542,15 @@ static void daemon_dump_matches( lst_List list )
 {
    lst_Position p;
    dictWord     *dw;
-   const char   *previous = NULL;
+   const char   *prevword = NULL;
+   dictDatabase *prevdb   = NULL;
    
    daemon_mime();
    LST_ITERATE(list,p,dw) {
-      if (previous && !strcmp(previous,dw->word)) continue;
-      previous = dw->word;
+      if (prevdb == dw->database && prevword && !strcmp(prevword,dw->word))
+	  continue;
+      prevword = dw->word;
+      prevdb   = dw->database;
       daemon_printf( "%s \"%s\"\n", dw->database->databaseName, dw->word );
    }
    daemon_printf( ".\n" );
@@ -556,10 +562,10 @@ static void daemon_banner( void )
 
    time(&t);
 
-   sprintf( daemonStamp, "<%d.%d.%ld@%s>", 
+   sprintf( daemonStamp, "<%d.%d.%lu@%s>", 
 	    _dict_forks,
 	    getpid(),
-	    t,
+	    (long unsigned)t,
 	    net_hostname() );
    daemon_printf( "%d %s %s <auth.mime> %s\n",
 		  CODE_HELLO,
