@@ -1,10 +1,10 @@
 /* dict.c -- 
- * Created: Fri Mar 28 19:16:29 1997 by faith@cs.unc.edu
- * Revised: Sun Feb 15 22:44:18 1998 by faith@acm.org
- * Copyright 1997, 1998 Rickard E. Faith (faith@acm.org)
+ * Created: Fri Mar 28 19:16:29 1997 by faith@dict.org
+ * Revised: Fri Dec 22 08:50:05 2000 by faith@dict.org
+ * Copyright 1997, 1998, 1999, 2000 Rickard E. Faith (faith@dict.org)
  * This program comes with ABSOLUTELY NO WARRANTY.
  * 
- * $Id: dict.c,v 1.18 1998/02/16 03:48:15 faith Exp $
+ * $Id: dict.c,v 1.26 2001/01/01 13:52:20 hilliard Exp $
  * 
  */
 
@@ -15,7 +15,7 @@
 extern int         yy_flex_debug;
        lst_List    dict_Servers;
        const char  *dict_pager;
-       FILE        *dict_output = stdout;
+       FILE        *dict_output;
 
 #define BUFFERSIZE  2048
 #define PIPESIZE     256
@@ -139,7 +139,9 @@ static void client_open_pager( void )
 				/* default */
    dict_output = stdout;
 				/* use an empty string to avoid paging */
-   if ((dict_pager || (dict_pager = getenv("PAGER"))) && *dict_pager) {
+   if ((dict_pager || (dict_pager = getenv("PAGER")))
+       && *dict_pager
+       && strcmp(dict_pager, "-")) {
       PRINTF(DBG_VERBOSE,("Using \"%s\" as pager\n",dict_pager));
       pr_open( dict_pager, PR_CREATE_STDIN, &infd, NULL, NULL );
       dict_output = fdopen( infd, "w" );
@@ -195,17 +197,6 @@ static void client_print_matches( lst_List l, int html, int flag,
       }
    }
 
-   if (flag) {
-      if (html) fprintf( dict_output, "<H2>" );
-      if (count)
-	 fprintf( dict_output,
-		  "%d match%s found", count, count == 1 ? "" : "es" );
-      else
-	 fprintf( dict_output, "No matches found for \"%s\"", word );
-      if (html) fprintf( dict_output, "</H2>\n" );
-      else      fprintf( dict_output, "\n" );
-   }
-
    if (!l) return;
 
    last = NULL;
@@ -237,6 +228,7 @@ static void client_print_matches( lst_List l, int html, int flag,
 	 fprintf( dict_output, "  %s", arg_get(a,1) );
 	 pos += len + 2;
       }
+      arg_destroy(a);
    }
    fprintf( dict_output, "\n" );
 }
@@ -255,6 +247,7 @@ static void client_print_listed( lst_List l, int html )
 	 err_internal( __FUNCTION__,
 		       "SHOW command didn't return 2 args: \"%s\"\n", e );
       fprintf( dict_output, "  %-10.10s %s\n", arg_get(a,0), arg_get(a,1) );
+      arg_destroy(a);
    }
    if (html) fprintf( dict_output, "</PRE>\n" );
 }
@@ -809,13 +802,15 @@ static void process( int html )
 	 expected = cmd_reply.retcode;
 	 break;
       case CMD_WIND:
-	 if (cmd_reply.matches) {
+	  if (cmd_reply.matches) {
 	    if (!cmd_reply.data)
 	       err_internal( __FUNCTION__,
 			     "%d matches, but no list\n", cmd_reply.matches );
 	    for (i = cmd_reply.matches; i > 0; --i) {
 	       const char *line = lst_nth_get( cmd_reply.data, i );
 	       arg_List   a = arg_argify( line, 0 );
+	       const char *orig, *s;
+	       char       *escaped, *d;
 	       if (arg_count(a) != 2)
 		  err_internal( __FUNCTION__,
 				"MATCH command didn't return 2 args: \"%s\"\n",
@@ -824,10 +819,24 @@ static void process( int html )
 					      str_find(arg_get(a,0)),
 					      str_copy(arg_get(a,1)),
 					      0 ) );
+				/* Escape " and \ in word before sending */
+	       orig    = arg_get(a,1);
+	       escaped = xmalloc(strlen(orig) * 2 + 1);
+	       for (s = orig, d = escaped; *s;) {
+		   switch (*s) {
+		   case '"':
+		   case '\\':
+		       *d++ = '\\';
+		   default:
+		       *d++ = *s++;
+		   }
+	       }
+	       *d++ = '\0';
 	       prepend_command( make_command( CMD_DEFINE,
 					      str_find(arg_get(a,0)),
-					      str_copy(arg_get(a,1)),
+					      str_copy(escaped),
 					      0 ) );
+	       xfree(escaped);
 	       arg_destroy(a);
 	    }
 	    client_free_text( cmd_reply.data );
@@ -931,7 +940,7 @@ static const char *id_string( const char *id )
 static const char *client_get_banner( void )
 {
    static char       *buffer= NULL;
-   const char        *id = "$Id: dict.c,v 1.18 1998/02/16 03:48:15 faith Exp $";
+   const char        *id = "$Id: dict.c,v 1.26 2001/01/01 13:52:20 hilliard Exp $";
    struct utsname    uts;
    
    if (buffer) return buffer;
@@ -944,10 +953,10 @@ static const char *client_get_banner( void )
    return buffer;
 }
 
-static void banner( void )
+static void banner( FILE *out_stream )
 {
-   fprintf( stderr, "%s\n", client_get_banner() );
-   fprintf( stderr,
+   fprintf( out_stream , "%s\n", client_get_banner() );
+   fprintf( out_stream,
 	    "Copyright 1997, 1998 Rickard E. Faith (faith@cs.unc.edu)\n" );
 }
 
@@ -971,42 +980,43 @@ static void license( void )
    0 };
    const char        **p = license_msg;
    
-   banner();
-   while (*p) fprintf( stderr, "   %s\n", *p++ );
+   banner ( stdout );
+   while (*p) fprintf( stdout, "   %s\n", *p++ );
 }
     
-static void help( void )
+static void help( FILE *out_stream )
 {
    static const char *help_msg[] = {
-      "-h --host <server>      specify server",
-      "-p --port <service>     specify port",
-      "-d --database <dbname>  select a database to search",
-      "-m --match              match instead of define",
-      "-s --strategy           strategy for matching or defining",
-      "-c --config <file>      specify configuration file",
-      "-C --nocorrect          disable attempted spelling correction",
-      "-D --dbs                show available databases",
-      "-S --strats             show available search strategies",
-      "-H --serverhelp         show server help",
-      "-i --info <dbname>      show information about a database",
-      "-I --serverinfo         show information about the server",
-      "-a --noauth             disable authentication",
-      "-u --user <username>    username for authentication",
-      "-k --key <key>          shared secret for authentication",
-      "-V --version            display version information",
-      "-L --license            display copyright and license information",
-      "   --help               display this help",
-      "-v --verbose            be verbose",
-      "-r --raw                trace raw transaction",
-      "   --debug <flag>       set debugging flag",
-      "   --html               output HTML format",
-      "   --pipesize <size>    specify buffer size for pipelining (256)",
-      "   --client <text>      additional text for client command",
+      "-h --host <server>        specify server",
+      "-p --port <service>       specify port",
+      "-d --database <dbname>    select a database to search",
+      "-m --match                match instead of define",
+      "-s --strategy <strategy>  strategy for matching or defining",
+      "-c --config <file>        specify configuration file",
+      "-C --nocorrect            disable attempted spelling correction",
+      "-D --dbs                  show available databases",
+      "-S --strats               show available search strategies",
+      "-H --serverhelp           show server help",
+      "-i --info <dbname>        show information about a database",
+      "-I --serverinfo           show information about the server",
+      "-a --noauth               disable authentication",
+      "-u --user <username>      username for authentication",
+      "-k --key <key>            shared secret for authentication",
+      "-V --version              display version information",
+      "-L --license              display copyright and license information",
+      "   --help                 display this help",
+      "-v --verbose              be verbose",
+      "-r --raw                  trace raw transaction",
+      "-P --pager program        specify program to use as pager (- for none)",
+      "   --debug <flag>         set debugging flag",
+      "   --html                 output HTML format",
+      "   --pipesize <size>      specify buffer size for pipelining (256)",
+      "   --client <text>        additional text for client command",
       0 };
    const char        **p = help_msg;
 
-   banner();
-   while (*p) fprintf( stderr, "%s\n", *p++ );
+   banner( out_stream );
+   while (*p) fprintf( out_stream, "%s\n", *p++ );
 }
 
 int main( int argc, char **argv )
@@ -1056,12 +1066,13 @@ int main( int argc, char **argv )
       { "raw",        0, 0, 'r' },
       { "pager",      1, 0, 'P' },
       { "debug",      1, 0, 502 },
-      { "html",       0, 0, 503 },
+      /*      { "html",       0, 0, 503 },*/
       { "pipesize",   1, 0, 504 },
       { "client",     1, 0, 505 },
       { 0,            0, 0,  0  }
    };
 
+   dict_output = stdout;
    maa_init(argv[0]);
 
    dbg_register( DBG_VERBOSE, "verbose" );
@@ -1092,7 +1103,7 @@ int main( int argc, char **argv )
       case 'a': doauth = 0;                            break;
       case 'u': user = optarg;                         break;
       case 'k': key = optarg;                          break;
-      case 'V': banner(); exit(1);                     break;
+      case 'V': banner( stdout ); exit(1);             break;
       case 'L': license(); exit(1);                    break;
       case 'v': dbg_set( "verbose" );                  break;
       case 'r': dbg_set( "raw" );                      break;
@@ -1101,17 +1112,16 @@ int main( int argc, char **argv )
       case 504: client_pipesize = atoi(optarg);        break;
       case 503: ++html;                                break;
       case 502: dbg_set( optarg );                     break;
-      case 501:					      
-      default:  help(); exit(1);                      break;
+      case 501:	help( stdout );	exit(1);               break;	      
+      default:  help( stderr ); exit(1);               break;
       }
 
    if (optind == argc && (!(function & ~(DEFINE|MATCH)))) {
-      banner();
+      banner( stderr );
       fprintf( stderr, "Use --help for help\n" );
       exit(1);
    }
 
-   if (client_pipesize < 0)       client_pipesize = 0;
    if (client_pipesize > 1000000) client_pipesize = 1000000;
    
    if (dbg_test(DBG_PARSE))     prs_set_debug(1);
