@@ -1,7 +1,8 @@
+
 /* dictP.h -- 
- * Created: Fri Mar  7 10:54:05 1997 by faith@cs.unc.edu
- * Revised: Mon Mar 10 10:50:04 1997 by faith@cs.unc.edu
- * Copyright 1997 Rickard E. Faith (faith@cs.unc.edu)
+ * Created: Fri Mar  7 10:54:05 1997 by faith@dict.org
+ * Revised: Fri Dec 22 06:06:33 2000 by faith@dict.org
+ * Copyright 1997, 1999, 2000 Rickard E. Faith (faith@dict.org)
  * This program comes with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -18,7 +19,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 675 Mass Ave, Cambridge, MA 02139, USA.
  * 
- * $Id: dictP.h,v 1.3 1997/03/10 21:46:55 faith Exp $
+ * $Id: dictP.h,v 1.5 2000/12/22 14:15:25 faith Exp $
  * 
  */
 
@@ -37,6 +38,10 @@
 #ifdef __GNUC__
 # define alloca __builtin_alloca
 #else
+# if defined(__svr4__) && defined(__sgi__) && !HAVE_ALLOCA_H /* IRIX */
+#  undef HAVE_ALLOCA_H
+#  define HAVE_ALLOCA_H 1
+# endif
 # if HAVE_ALLOCA_H
 #  include <alloca.h>
 # else
@@ -44,8 +49,10 @@
  #pragma alloca
 #  else
 #   ifndef alloca /* predefined by HP cc +Olibcalls */
+#   if !defined(__svr4__) && !defined(__sgi__)	/* not on IRIX */
 char *alloca ();
 #   endif
+#   endif
 #  endif
 # endif
 #endif
