@@ -1,10 +1,10 @@
 /* dictd.c -- 
- * Created: Fri Feb 21 20:09:09 1997 by faith@cs.unc.edu
- * Revised: Sun Jul  5 21:08:46 1998 by faith@acm.org
- * Copyright 1997, 1998 Rickard E. Faith (faith@acm.org)
+ * Created: Fri Feb 21 20:09:09 1997 by faith@dict.org
+ * Revised: Wed Nov  8 05:57:21 2000 by faith@dict.org
+ * Copyright 1997-2000 Rickard E. Faith (faith@dict.org)
  * This program comes with ABSOLUTELY NO WARRANTY.
  * 
- * $Id: dictd.c,v 1.39 1998/07/06 01:35:40 faith Exp $
+ * $Id: dictd.c,v 1.42 2000/11/08 11:00:45 faith Exp $
  * 
  */
 
@@ -14,6 +14,12 @@
 #define MAXPROCTITLE 2048       /* Maximum amount of proc title we'll use. */
 #undef MIN
 #define MIN(a,b) ((a)<(b)?(a):(b))
+#ifndef UID_NOBODY
+#define UID_NOBODY  65534
+#endif
+#ifndef GID_NOGROUP
+#define GID_NOGROUP 65534
+#endif
 
 extern int        yy_flex_debug;
 static int        _dict_daemon;
@@ -54,7 +60,7 @@ void dict_setproctitle( const char *format, ... )
       err_fatal( __FUNCTION__, "buffer overflow (%d)\n", len );
    buf[ MIN(_dict_argvlen,MAXPROCTITLE) - 1 ] = '\0';
    strcpy( _dict_argvstart, buf );
-   memset( _dict_argvstart+len, ' ', _dict_argvlen-len );
+   memset( _dict_argvstart+len, 0, _dict_argvlen-len );
 }
 
 const char *dict_format_time( double t )
@@ -91,7 +97,7 @@ const char *dict_format_time( double t )
 
 static void reaper( int dummy )
 {
-#if defined(__sparc__) && defined(__svr4__)
+#if defined(__osf__) || (defined(__sparc__) && defined(__svr4__))
    int        status;
 #else
    union wait status;
@@ -102,7 +108,7 @@ static void reaper( int dummy )
       ++_dict_reaps;
       
       if (flg_test(LOG_SERVER))
-         log_info( ":I: Reaped %d%s%s\n",
+         log_info( ":I: Reaped %d%s\n",
                    pid,
                    _dict_daemon ? " IN CHILD": "" );
    }
@@ -376,7 +382,7 @@ const char *dict_get_banner( int shortFlag )
 {
    static char    *shortBuffer = NULL;
    static char    *longBuffer = NULL;
-   const char     *id = "$Id: dictd.c,v 1.39 1998/07/06 01:35:40 faith Exp $";
+   const char     *id = "$Id: dictd.c,v 1.42 2000/11/08 11:00:45 faith Exp $";
    struct utsname uts;
    
    if (shortFlag && shortBuffer) return shortBuffer;
@@ -464,6 +470,19 @@ static void set_minimal( void )
    flg_set("-min");
 }
 
+static void release_root_privileges( void )
+/* At the spring 1999 Linux Expo in Raleigh, Rik Faith told me that he
+ * did not want dictd to be allowed to run as root for any reason.
+ * This patch irrevocably releases root privileges.  -- Kirk Hilliard
+ */
+{
+   if (geteuid() == 0) {
+      setgid(GID_NOGROUP);
+      initgroups("nobody", GID_NOGROUP);
+      setuid(UID_NOBODY);
+   }
+}
+
 int main( int argc, char **argv, char **envp )
 {
    int                childSocket;
@@ -503,6 +522,7 @@ int main( int argc, char **argv, char **envp )
       { 0,          0, 0,  0  }
    };
 
+   release_root_privileges();
    maa_init(argv[0]);
 
    flg_register( LOG_SERVER,    "server" );
@@ -636,28 +656,7 @@ int main( int argc, char **argv, char **envp )
    fflush(stdout);
    fflush(stderr);
 
-   if (detach) {
-      FILE *str;
-
-      if (!forceStartup && (str = fopen(DICTD_PID_FILE, "r"))) {
-	 char buf[80];
-	 if (fread(buf,1,80,str) > 0) {
-	    buf[strlen(buf)-1] = '\0'; /* Kill newline */
-	    err_fatal( __FUNCTION__,
-		       "dictd already running with pid %s\n"
-		       "   remove %s or use -f to force startup\n",
-		       buf, DICTD_PID_FILE );
-	 }
-	 fclose(str);
-      }
-      
-      net_detach();
-      
-      if ((str = fopen(DICTD_PID_FILE, "w"))) {
-	 fprintf( str, "%d\n", (int)getpid() );
-	 fclose( str );
-      }
-   }
+   if (detach) net_detach();
 
    masterSocket = net_open_tcp( service, depth );
    
